/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef ParNCLNetCdf_h
#define ParNCLNetCdf_h

#ifdef NIO_LIB_ONLY
#include "niohlu.h"
#include "nioNresDB.h"
#else
#include <ncarg/hlu/hlu.h>
#include <ncarg/hlu/NresDB.h>
#endif
#include "defs.h"
#include "NclDataDefs.h"
#include "NclFileInterfaces.h"
#include "NclVar.h"
#include "VarSupport.h"
#include "NclData.h"

#include   <stdio.h>
#include   <sys/types.h>
#include   <sys/stat.h>
#include   <fcntl.h>
#include   <unistd.h>
#include   <string.h>
#include   <strings.h>
#include   <dirent.h>
#include   <stdlib.h>

#include <netcdf.h>
#include <math.h>

typedef struct _PNNCVarInqRec PNNCVarInqRec;
typedef struct _PNNCDimInqRec PNNCDimInqRec;
typedef struct _PNNCAttInqRec PNNCAttInqRec;
typedef struct _PNNCVarInqRecList PNNCVarInqRecList;
typedef struct _PNNCDimInqRecList PNNCDimInqRecList;
typedef struct _PNNCAttInqRecList PNNCAttInqRecList;

struct _PNNCVarInqRecList {
	PNNCVarInqRec *var_inq;
	PNNCVarInqRecList *next;
};

struct _PNNCDimInqRecList {
	PNNCDimInqRec *dim_inq;
	PNNCDimInqRecList *next;
};

struct _PNNCAttInqRecList {
	PNNCAttInqRec *att_inq;
	PNNCAttInqRecList *next;
};

typedef struct _PNNCVarDimInfo{
	NclQuark name;
	int dim_id;
	ng_size_t size;
} PNNCVarDimInfo;

struct _PNNCVarInqRec {
	int varid;
	NclQuark name;
	nc_type data_type;
	int n_dims;
	/* FIXME: Is there a better way than holding all info abt var dims here ? */
	PNNCVarDimInfo dim_info[MAX_VAR_DIMS];
	int n_atts;
	PNNCAttInqRecList *att_list;
};

struct _PNNCDimInqRec {
	int dimid;
	int is_unlimited;
	NclQuark name;
	ng_size_t size;
};

struct _PNNCAttInqRec {
	int att_num;
	NclQuark name;
	int varid;
	NclBasicDataTypes data_type;
	int len;
	void *value;
	/* int virtual; */
};

#endif
